namespace K3D.Geometry
{
	using System;
	using System.Text;


	/// <summary>
	/// Polygon
	/// Represents a polygon defined by 3 or more Points
	/// </summary>
	public sealed class Polygon
	{
		// ------------------------------------------------------------------------------
		// Construction
      
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="vertices">Array int indexes into the layers point set</param>
		public Polygon(int[] vertices)
		{
			m_vertices = vertices;
		}
      
      
		// ------------------------------------------------------------------------------
		// Public methods
      
		/// <summary>
		/// Return the polygon as a string
		/// </summary>
		/// <returns>Polygon as a string for output</returns>
		public override string ToString()
		{
			StringBuilder buf = new StringBuilder(128);

			buf.Append("Polygon (" + m_vertices.Length + " points): ");
			for (int iPoints=0; iPoints<m_vertices.Length; iPoints++)
			{
				buf.Append(m_vertices[iPoints].ToString());
				if (iPoints < m_vertices.Length - 1)
				{
					buf.Append(", ");
				}
			}
         
			return buf.ToString();
		}
      
		/// <summary>
		/// Gets the number of verticies in this polygon
		/// </summary>
		public int VertexCount
		{
			get
			{
				return m_vertices.Length;
			}
		}

      /// <summary>
      /// Get the vertex indexes for this polygon
      /// </summary>
      public int[] Vertices
      {
         get
         {
            return m_vertices;
         }
      }

		/// <summary>
		/// The index into the surface table for this polygon
		/// </summary>
		public int SurfaceIndex
		{
			get
			{
				return m_surface;
			}
			set
			{
				m_surface = value;
			}
		}

      /// <summary>
      /// The normal vector for this polygon
      /// </summary>
      public Point Normal
      {
         get
         {
            return m_normal;
         }
         set
         {
            m_normal = value;
         }
      }

      /// <summary>
      /// Normals for each vertex of the polygon
      /// </summary>
      public Point[] VertexNormals
      {
         get
         {
            return m_vertexNormals;
         }
         set
         {
            m_vertexNormals = value;
         }
      }

		/// <summary>
		/// Whether to force smoothing on for this polygon
		/// </summary>
		public bool ForceSmoothing
		{
			get
			{
				return m_forceSmooth;
			}
			set
			{
				m_forceSmooth = value;
			}
		}

      /// <summary>
      /// Whether to ignore this polygon during rendering
      /// </summary>
      public bool Ignore
      {
         get
         {
            return m_bIgnore;
         }
         set
         {
            m_bIgnore = value;
         }
      }
      
      
		// ------------------------------------------------------------------------------
		// Private data
      
      private bool m_bIgnore = false;
		
		private int[] m_vertices = null;
      private Point m_normal;
      private Point[] m_vertexNormals = null;
		
		private int m_surface = 0;
		
		private bool m_forceSmooth = false;
	}
}
