/*
 * K3D Object Viewer Copyright (C) 1998-2005 Kevin Roast
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation;
 * either version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 */
namespace K3D
{
	using System;
	using System.Collections;
	using System.Drawing;
	using System.Drawing.Imaging;
	using System.Text;
	using System.Threading;
	using System.Windows.Forms;
   using System.Diagnostics;

	using Tao.OpenGl;
	using Tao.Platform.Windows;


	/// <summary>
	/// K3DGLControl
	/// This abstract class represents an OpenGL aware control
	/// </summary>
	public abstract class K3DGLControl : SimpleOpenGlControl
	{
		/// <summary>
		/// Reset the projection matrix when the window size is changed
		/// </summary>
		protected override void OnSizeChanged(EventArgs e)
		{
			base.OnSizeChanged(e);
         
         //if (K3DGLForm.DEBUG) Trace.WriteLine("K3DGLControl::OnSizeChanged()");

			Size s = Size;
			double aspect_ratio = (double)s.Width/(double)s.Height;
         Gl.glViewport(0, 0, s.Width, s.Height);
         
			// select the projection matrix and clear it
			Gl.glMatrixMode(Gl.GL_PROJECTION);
			Gl.glLoadIdentity();

         // field of view in degree, aspect ratio, Z near, Z far
         Glu.gluPerspective(30.0, -aspect_ratio, 1.0, 1000.0);
         
			// switch back to the modelview matrix and clear it
         // eye is at, center is at, up is i0n
         Glu.gluLookAt(0.0,0.0,100.0, 0.0,0.0,0.0, 0.0,1.0,0.0);
			Gl.glMatrixMode(Gl.GL_MODELVIEW);
			Gl.glLoadIdentity();
		}
      
		/// <summary>
		/// Handles swapping of buffers and calls GLDraw() to render
		/// </summary>
		protected override void OnPaint(PaintEventArgs e) 
		{
			if (IsDisposed || Disposing)
			{
				return;
			}
         
			GLDraw();
			Gl.glFinish();
			SwapBuffers();
		}
   
		/// <summary>
		/// Called when GL primitives are ready to be drawn
		/// </summary>
		public abstract void GLDraw();
      
		/// <summary>
		/// Helper to load a bitmap image
		/// </summary>
		protected Bitmap loadImage(string dir, string name)
		{
			return new Bitmap(dir + '/' + name);
		}
	}
}
