namespace K3D.ThreeD.Lightwave
{
	using System;
	using System.Collections;
	using System.Text;

   using K3D.Geometry;


	/// <summary>
	/// A single Lightwave object layer.
	/// </summary>
	public sealed class LightwaveLayer
	{
		// ------------------------------------------------------------------------------
		// Public Properties

		/// <summary>
		/// Polygons for this layer
		/// </summary>
		public Polygon[] Polygons
		{
			get
			{
				if (m_polygons == null && (m_rawPolys != null || m_subPatchPolys != null))
				{
					if (m_subPatchPolys != null)
					{
						m_polygons = (Polygon[])m_subPatchPolys.ToArray(typeof(Polygon));
						m_subPatchPolys = null;
					}
					else
					{
						m_polygons = (Polygon[])m_rawPolys.ToArray(typeof(Polygon));
						m_rawPolys = null;
					}
				}
				
				return m_polygons;
			}
		}

		/// <summary>
		/// Sub-patch polygons for this layer
		/// </summary>
		public ArrayList SubPatchPolygons
		{
			get
			{
				return m_subPatchPolys;
			}
			set
			{
				m_subPatchPolys = value;
				m_lastPolys = value;
			}
		}

		/// <summary>
		/// Raw polygons for this layer
		/// </summary>
		public ArrayList RawPolygons
		{
			get
			{
				return m_rawPolys;
			}
			set
			{
				m_rawPolys = value;
				m_lastPolys = value;
			}
		}

		/// <summary>
		/// Last added polys (for applying surface tags)
		/// </summary>
		public ArrayList LastAddedPolygons
		{
			get
			{
				return m_lastPolys;
			}
		}

		/// <summary>
		/// Points for this layer
		/// </summary>
		public Point[] Points
		{
			get
			{
				return m_points;
			}
			set
			{
				m_points = value;
			}
		}

		public override string ToString()
		{
			StringBuilder buf = new StringBuilder(256);

			buf.Append(" Points:\n");
			foreach (Point point in m_points)
			{
				buf.Append("\t").Append(point.ToString()).Append("\n");
			}

			buf.Append("Polygons:\n");
			foreach (Polygon poly in m_polygons)
			{
				buf.Append("\t").Append(poly.ToString()).Append("\n");
			}

			return buf.ToString();
		}


		// ------------------------------------------------------------------------------
		// Private data

		private ArrayList m_rawPolys = null;
		private ArrayList m_subPatchPolys = null;
		private ArrayList m_lastPolys = null;

		private Polygon[] m_polygons = null;
		private Point[]   m_points = new Point[0];
	}
}
