namespace K3D.ThreeD.Lightwave
{
   using System;
   using System.Collections;

	using K3D.Geometry;


	/// <summary>
	/// Represents the list of polys against surfaces that a single point
	/// is referenced in.
	/// </summary>
	public sealed class UsedInPolySurfaceCollection
	{
      /// <summary>
      /// Add a poly index with surface index to the used-in list
      /// </summary>
      /// <param name="polyIndex"></param>
      /// <param name="surfaceIndex"></param>
      public void AddUsedIn(int polyIndex, int surfaceIndex)
      {
         if (m_itemIndex == m_usedPolyIndexes.Length)
         {
            // resize arrays by hand to fit and copy to new arrays
            int[] temp = new int[m_itemIndex << 1];
            Array.Copy(m_usedPolyIndexes, temp, m_itemIndex);
            m_usedPolyIndexes = temp;
            temp = new int[m_itemIndex << 1];
            Array.Copy(m_usedSurfaceIndexes, temp, m_itemIndex);
            m_usedSurfaceIndexes = temp;
         }
         
         m_usedPolyIndexes[m_itemIndex] = polyIndex;
         m_usedSurfaceIndexes[m_itemIndex] = surfaceIndex;

         m_itemIndex++;
      }

      /// <summary>
      /// Return the poly indexes with the specified surface index
      /// that reference this point.
      /// </summary>
      /// <param name="surfaceIndex"></param>
      /// <returns></returns>
      public int[] GetPolysWithSurface(int surfaceIndex)
      {
         // walk the list to find this poly with this index
         int foundCount = 0;
         for (int i=0; i<m_itemIndex; i++)
         {
            if (m_usedSurfaceIndexes[i] == surfaceIndex)
            {
               foundCount++;
            }
         }
         
         int[] foundPolys = new int[foundCount];
         
         foundCount = 0;
         for (int i=0; i<m_itemIndex; i++)
         {
            if (m_usedSurfaceIndexes[i] == surfaceIndex)
            {
               foundPolys[foundCount++] = m_usedPolyIndexes[i];
            }
         }
         
         return foundPolys;
      }

      private int m_itemIndex = 0;
      private int[] m_usedPolyIndexes = new int[4];
      private int[] m_usedSurfaceIndexes = new int[4];
	}
}
